"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.some = exports.Some = void 0;
const result_1 = require("../result");
const equality_1 = require("../equality");
const none_1 = require("./none");
const iClonable_1 = require("../clone/iClonable");
const iDecodable_1 = require("../codable/iDecodable");
const iEncodable_1 = require("../codable/iEncodable");
class Some {
    constructor(value) {
        this.value = value;
        this.isDefined = true;
        this.isEmpty = false;
    }
    get get() {
        return this.value;
    }
    getOrElse(defaultValue) {
        return this.value;
    }
    orElse(defaultValue) {
        return this;
    }
    map(f) {
        return new Some(f(this.value));
    }
    flatMap(f) {
        return f(this.value);
    }
    equals(compared, comparisonFunc) {
        if (compared instanceof none_1.None)
            return false;
        if (!(compared instanceof Some)) {
            return (0, equality_1.equals)(this.value, compared);
        }
        if (comparisonFunc === undefined) {
            return (0, equality_1.equals)(this.value, compared.value);
        }
        return comparisonFunc(this.value, compared.value);
    }
    fold(left, right) {
        return right(this.value);
    }
    unwrap(or) {
        return this.value;
    }
    toResult(_) {
        return (0, result_1.success)(this.value);
    }
    get clone() {
        if ((0, iClonable_1.isClonable)(this.value)) {
            return some(this.value.clone);
        }
        if (typeof this.value === "object") {
            return some(Object.create(this.value));
        }
        return some(this.value);
    }
    get encode() {
        if ((0, iEncodable_1.isEncodable)(this.value)) {
            return this.value.encode;
        }
        return this.value;
    }
    decode(json) {
        if (json == null)
            return (0, none_1.none)();
        if ((0, iDecodable_1.isDecodable)(this.value)) {
            return some(this.value.decode(json));
        }
        return some(json);
    }
}
exports.Some = Some;
function some(value) {
    return new Some(value);
}
exports.some = some;
//# sourceMappingURL=some.js.map